//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name           : main_AdcPdc.c
//* Object              : main application written in C
//* Creation            : S.CADENE   01/September/2004
//* Vers                : V1.0
//* Aim : What is the minimum program in order to perform multi-shot conversion
//*       by using hardware triggering via Timer Output ?
//*       Here is the program to complet this task.
//*       This software does not manage Overrun status bit.
//*       Used resource : ADC0 pin, internal SRAM area Buffers, TIOA0 Timer
//*                       output.
//*----------------------------------------------------------------------------

// Include Standard files
#include "Board.h"

//***********************************************************
// Constant definition used for the timer
//***********************************************************

/* Clock Sources for the timer channel */
//***********************************************************
#define TC_CLKS_MCK_DIV_2        0x0    // called TIMER_CLOCK1 in datasheet
#define TC_CLKS_MCK_DIV_8        0x1    // called TIMER_CLOCK2 in datasheet
#define TC_CLKS_MCK_DIV_32       0x2    // called TIMER_CLOCK3 in datasheet
#define TC_CLKS_MCK_DIV_128      0x3    // called TIMER_CLOCK4 in datasheet
#define TC_CLKS_MCK_DIV_1024     0x4    // called TIMER_CLOCK5 in datasheet
#define TC_CLKS_XC0              0x5    // called XC0 in datasheet
#define TC_CLKS_XC1              0x6    // called XC1 in datasheet
#define TC_CLKS_XC2              0x7    // called XC2 in datasheet

/* Targeted Frequency Output from the Timer which will be the 
sampling frequency 
Reminder: The master clock frequency is 47.923200 MHz
With Master clock /2 (TCCLKS = 0) we can switch between 365.625 Hz up to 23.9616 MHz
With Master clock /8 (TCCLKS = 1) we can switch between 91.40625 Hz up to 5.9904 MHz
With Master clock /32 (TCCLKS = 2) we can switch between  22.8515.. Hz up to 1.4976 MHz
With Master clock /128 (TCCLKS = 3) we can switch between 5.71289 Hz up to 374.4 kHz
With Master clock /1024 (TCCLKS = 4) we can switch between 0.71411... Hz up to 46800 Hz
*/
//*************************************************************************************
//                      CAPTURE AERA
// You have to capture the following define expression according to your application.
//

/* Select the targeted Timer ID that will be used to trig the ADC */
#define TIMER_ID                  12    // Timer 0
// #define TIMER_ID               13    // Timer 1
// #define TIMER_ID               14    // Timer 2

/* Select the ratio between master clock and the timer clock. According to this choice,
you have to modify TIMER_VALUE definition below */
#define   TC_MCK_DIV            4   // TCCLKS = 1 this field set the division between the
                                    // master clock and the source clock of the timer

/* Select the sampling frequency in hertz */
#define   SAMPLING_FREQUENCY    50  // In Hertz

/* Select the number of wanted sample */
#define   NSAMPLE    64

/* You have to capture, in this formula, 2, 8, 32, 128 or 1024 according to TC_MCK_DIV */
#define   TIMER_VALUE            (MCK/(1024*SAMPLING_FREQUENCY))

/* Targeted Pulse Width */
#define   PULSEWIDTH  20             // Pulse duration in term of Timer Clock Cycle(s)

//**************************************************************************************

/* Will be the targeted receive buffer for the PDC */
volatile unsigned short AdcBuffer[NSAMPLE*2];

//***********************************************************
// ADC field definition for the Mode Register 
//***********************************************************
/* Reminder            TRGEN    => Selection bewteen Software or hardware start of conversion
                       TRGSEL   => Relevant if previous field is set on hardware triggering
				0x0 << 1 (ADC) Selected TRGSEL = TIAO0
				0x1 << 1 (ADC) Selected TRGSEL = TIAO1
				0x2 << 1 (ADC) Selected TRGSEL = TIAO2
                       LOWRES   => 10-bit result if ths bit is cleared 0
                       SLEEP    => normal mode if ths is cleared
                       PRESCAL  => ADCclock = MCK / [(PRESCAL + 1)*2]
                       STARTUP  => Startup Time = [(STARTUP + 1)*8] / ADCclock
                       SHTIM    => Tracking time = (SHTIM + 1) / ADCclock
 */
#define   TRGEN    (0x1)    // Hardware triggering
#define   TRGSEL   (0x0)    // Use a Timer output signal (on rising edge) from TIOA0 (for this example)
#define   LOWRES   (0x0)    // 10-bit result output
#define   SLEEP    (0x0)    // Normal Mode
#define   PRESCAL  (0x4)    // Max value
#define   STARTUP  (0xc)    // This time period must be higher than 20 s and not 20 ms
#define   SHTIM    (0x2)    // Must be higher than 3 ADC clock cycles but depends on output
                            // impedance of the analog driver to the ADC input
/* Channel selection */
#define   CHANNEL  (0)      // Write the targeted channel with the first channel is 0 and the last 7
#define   CHANNEL1 (1)
//*--------------------------------------------------------------------------------------
//* Function Name       : ADCInit
//* Object              : Setting up the ADC and its associated PDC
//*--------------------------------------------------------------------------------------
void ADCInit (void)
{           
       /* Clear all previous setting and result */
       AT91F_ADC_SoftReset (AT91C_BASE_ADC);
       
       AT91F_PDC_Open (AT91C_BASE_PDC_ADC);
             
       /* First Step: Set the PDC buffer and counter */
       AT91F_PDC_ReceiveFrame ( AT91C_BASE_PDC_ADC ,
                                (char *) &AdcBuffer[0]       , NSAMPLE,
                                (char *) &AdcBuffer[NSAMPLE] , NSAMPLE);
       
        /* Second step: Set up by using ADC Mode register */
        AT91F_ADC_CfgModeReg (AT91C_BASE_ADC, (SHTIM << 24) | (STARTUP << 16) | (PRESCAL << 8) |
                             (SLEEP << 5) | (LOWRES <<4) | (TRGSEL << 1) | (TRGEN ) ) ;
       
        /* Third Step: Select the active channel */
        AT91F_ADC_EnableChannel (AT91C_BASE_ADC, (1<<CHANNEL));
        AT91F_ADC_EnableChannel (AT91C_BASE_ADC, (1<<CHANNEL1));
}
//*--------------------------------------------------------------------------------------
//* Function Name       : AT91F_TC_Open
//* Object              : Setting up Timer A
//*--------------------------------------------------------------------------------------
void AT91F_TC_Open (AT91PS_TC TC_pt, unsigned int Mode, unsigned int TimerId)
{
/****** First Step:  Enable the Timer's clock at PMC level */
    	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << TimerId ) ;

    /* In order to select TIOA0 we have to disable PA0 and select periph.B */
    /* Reminder:
    Peripheral Signal       Corresponding PIO Signal      Multiplexing betw. A or B
    TIOA0               => PA0                            =>Periph.B
    TIOB0               => PA1                            =>Periph.B
    TIOA1               => PA2                            =>Periph.B
    TIOB1               => PA3                            =>Periph.B
    TIOA2               => PA4                            =>Periph.B
    TIOB2               => PA5                            =>Periph.B */
    /* Allow the user to display the trig signal from the Timer Peripheral
       through a pin, but it is not mandatory (for the debug).*/   
        AT91F_PIO_CfgPeriph (AT91C_BASE_PIOA, 0 , AT91C_PA0_TIOA0);
      
/****** Second Step: Disable the clock and the interrupts */
	TC_pt->TC_CCR = AT91C_TC_CLKDIS ;
     
/****** Third Step: Set the Mode of the Timer Counter */
	TC_pt->TC_CMR = Mode ;

/****** Fourth Step: The RC will give you the signal period and (RC-RA) the slot width */ 
	TC_pt->TC_RC = TIMER_VALUE ;
        TC_pt->TC_RA = (TIMER_VALUE - PULSEWIDTH) ;

/****** Fifth Step:Start the timer counter */
	TC_pt->TC_CCR = (AT91C_TC_CLKEN |AT91C_TC_SWTRG ) ;
}
//*--------------------------------------------------------------------------------------
//* Function Name       : Main
//* Object              : Software entry point
//*--------------------------------------------------------------------------------------
int main(void)
{
      
/******* First step: ADC Init is now embedded in function to clarify the main code */
        ADCInit();
 
/******* Second Step: Setting up the Timer channel in order to valid hardware and start it */ 
        AT91F_TC_Open ( AT91C_BASE_TC0, (AT91C_TC_ASWTRG_CLEAR | AT91C_TC_ACPC_CLEAR |
                        AT91C_TC_ACPA_SET | AT91C_TC_LDRA_RISING | AT91C_TC_WAVE  |
                        AT91C_TC_WAVESEL_UP_AUTO | TC_MCK_DIV ), TIMER_ID);

/******* Main instructions to fetch the NSAMPLE results *************************/
        /* Waiting the end of PDC transfer from both buffers */
        while( !(AT91F_PDC_IsRxEmpty (AT91C_BASE_PDC_ADC)) );
              
        /* infinit loop to avoid to get out*/
        while (1);
}
//* End
