/*-----------------------------------------------------------------------------
*         ATMEL Microcontroller Software Support  -  ROUSSET  -
*------------------------------------------------------------------------------
* The software is delivered "AS IS" without warranty or condition of any
* kind, either express, implied or statutory. This includes without
* limitation any warranty or condition with respect to merchantability or
* fitness for any particular purpose, or against the infringements of
* intellectual property rights of others.
*------------------------------------------------------------------------------
* File Name           : All-in-one-Demo.c
* Object              : main application written in C.
*                     : Demo with the following Devices
*                     : - Graphics LCD (PCF8558)
*                       - Maxim/Dallas DS1337 RTC
*                       - NS LM75 Digital Temperature Sensor
*                       - ATMEL AT24C1024 TWI EEPROM
*
* Version | mm | dd | yy | author :
*  1.0      12   27   06    PFi   : Creation
*------------------------------------------------------------------------------
* 1)Description:
*
* This Demo uses all the drivers from the other standalone projects to make a
* All-In-One Demo.
* The Demo display the temperature in degree Celsius and Fahrenheit,
* the GMT and Local time after setting the DS1337 with your PC's system time.
* The ATMEL TWI Serial EEPROM is used to store small 1 bit per pixel bitmaps.
* To switch between bitmaps and animated demos, use the SW4.
* To come back to time, date and temperature display use SW3.
* Note that the images must be downloaded beforehand into the TWI EEPROM
* with the EEPROM-Image-Loader Project. This will avoid to use memory space to
* store the images.
*
*  For further details check the following Application note
*  "AT91-AN01: Using the Two-wire interface (TWI) in
*  Master Mode on AT91SAM Microcontrollers."
*  www.atmel.com/dyn/products/app_notes.asp?family_id=605
*
* - By default IAR compiler use the Tiny printf and scanf library format.
*   To be able to make print of float variables, you must set the library
*   format to large.
*
* Remarks:
* No pull ups are mounted on the board on TWD and TWCK. 2.2K Pull up used
* By default IAR compiler use the Tiny printf and scanf library format.
* To be able to make print of float variables, you must set the library
* format to large.
*
* This example runs in polling mode.and makes no use of IRQs. To have further
* details and implementation example about using IRQs, refer to any Interrupt
* project examples provided in Getting Started available on
* www.atmel.com/dyn/products/app_notes.asp?family_id=605*
*----------------------------------------------------------------------------*/

/* Include Standard LIB  files */
#include "Board.h"
#include "common\lib_twi.h"
#include "time.h"
#include "rtc_ds1337\ds1337.h"
#include "atmel_at24c\at24c.h"
#include "glcd_batron\glcd_i2c_batron.h"
#include "lm75\lm75.h"
#include "math.h"

/*---------------------------  Constants definition -------------------------*/
#define TWI_BUS_CLOCK   400000 /* in Hz*/

#define LOCAL_TIME_OFFSET   1   /* OffSet from GMT Time */
#define SUMMER_TIME_OFFSET   1   /* winter / summer offset Time */

#define PIO_INTERRUPT_LEVEL 6
#define PIO_PIN_IRQ_MASK    (AT91C_PIO_PA2|AT91C_PIO_PA14|AT91C_PIO_PA15)

#define DISPLAY_DELAY   3 /* in sec. Delay between displaying messages on LCD */

/* Global varialbes and structure definition */
volatile int ReadRtc = 0 ;
volatile int RtcIrqCounter = 0 ;
volatile int Sw4IrqCounter = 0 ;
volatile int Sw3IrqCounter = 0 ;
volatile unsigned int TwiClock = TWI_BUS_CLOCK ;

volatile unsigned char StaticImagesNum;
volatile unsigned char AnimeImagesNum;

volatile char message[19];      /* Buffer for sprintf */

struct  DS1337 ptDS1337 ;
struct at24c ptAt24c1024 ;

struct tm *SystemTime ;

volatile unsigned int Next = 0 ;   /* Hold the next demo to call */
typedef void (*ptFunction)(void);  /* Pointer to a function */
ptFunction ptFunctionTable[4];     /* Array of Pointer to functions */
int IndexPtFunction = 0;

/* Available in Demo4All-in-one-Demo.c */
extern void AT91F_Demo1(void);
extern void AT91F_Demo2(void);
extern void AT91F_Demo3(void);
extern void AT91F_Demo4(void);

/* External functions */
extern void AT91F_WaitSecond (unsigned int seconde);
extern void AT91F_WaitMiliSecond (unsigned int MiliSeconds);
extern void AT91F_Wait_MicroSecond (unsigned int MicroSecond);

/* Global varialbes and structure definition */
/* Image descriptor that will be stored in page 0 of the EEPROM */
struct Image {
    unsigned char ImgNumber;
    unsigned char ImgVsize;
    unsigned char ImgHsize;
    unsigned char ImgPageStart;
    const unsigned char *Image;
    };

/* Declare an Image Table */
struct Image ImageTable[];

/* Animated bitmap descriptor */
struct AnimeImg {
    unsigned char AnimeImgNumber;
    unsigned char AnimeImgVsize;
    unsigned char AnimeImgHsize;
    unsigned char AnimeImgNumOfImage;
    unsigned char AnimeImgDelay;        /* delay between images in ms */
    unsigned char AnimeImgPageStart;
    unsigned char *Image[480];
    };

/* Declare an Image Table */
struct AnimeImg AnimeImgTable[];

/*-----------------------------------------------------------------------------
* Function Name       : void AT91F_Pio_C_Irq_Handler ( void )
* Object              : Irq C Handler
* Input Parameters    : none
* Output Parameters   : none
* Comments            :
-----------------------------------------------------------------------------*/
void AT91F_Pio_C_Irq_Handler ( void )
{

  unsigned int dummy, RtcIrqState,Sw4IrqState, SwState, State ;

  /* read the interrupt status reg of the PIO to acknoledge the IRQ */
  dummy = AT91C_BASE_PIOA->PIO_ISR;
  dummy = dummy;

  RtcIrqState = ((AT91C_BASE_PIOA->PIO_PDSR) & AT91C_PIO_PA2) >> 2 ;
  Sw4IrqState = ((AT91C_BASE_PIOA->PIO_PDSR) & AT91C_PIO_PA14) >> 14 ;
  SwState =  Sw4IrqState ;
  State = !SwState | !RtcIrqState ;

  if (!State) /* external RTC's IRQ */
  {
    Sw4IrqState = 1 ;
    RtcIrqCounter++;
    ReadRtc = 1;
  }

  if (!Sw4IrqState ) /* SW4 push button IRQ */
  {
    ReadRtc = 0;
    Sw4IrqCounter++;

    /* Stay here a little bit for debouncing */
    AT91F_WaitMiliSecond(150 ) ;

    IndexPtFunction++;

    if ( IndexPtFunction == 4)
      IndexPtFunction= 0;
  }

}

/*-----------------------------------------------------------------------------
* Function Name       : main
* Object              : Main function
* Input Parameters    : none
* Output Parameters   : TRUE
-----------------------------------------------------------------------------*/
int main( void )
{

  /* TWI AT24C1024 EEPROM variables */

  unsigned char ByteOffSet;

  /* Time Structures */
  time_t t = time(NULL);
  SystemTime = localtime(&t);

  /* GLCD and images variables */
  unsigned int ImgIndex, i, I2cDevice, LcdPage;
  unsigned int AnimeImgIndex;
  unsigned char ValidStaticImg=0;
  unsigned char ValidAnimeImg=0;

  ptFunctionTable[0]= &AT91F_Demo1;
  ptFunctionTable[1]= &AT91F_Demo2;
  ptFunctionTable[2]= &AT91F_Demo3;
  ptFunctionTable[3]= &AT91F_Demo4;

  /* Initialize the EEPROM descriptor */
  ptAt24c1024.PageSize = 256;
  ptAt24c1024.NumOfPage = 512;
  ptAt24c1024.EepromSize = 256*512;
  ptAt24c1024.SlaveAddress = AT24C_I2C_ADDRESS;
  ptAt24c1024.EepromName = AT24C1024;

  /* Make a bus recovery sequence in case of the transmission has been
  broken due to a power loss or reset. Since it is done with GPIO
  no needs to open the TWI bus */
  AT91F_AT24C_MemoryReset (AT91C_BASE_PIOA,
                             AT91C_ID_PIOA,
                             AT91C_PA3_TWD,
                             AT91C_PA4_TWCK);

  AT91F_Glcd_Reset (AT91C_BASE_PIOA,
                    AT91C_ID_PIOA,
                    AT91C_PA3_TWD,
                    AT91C_PA4_TWCK);

  /* Pointer to the AIC */
  AT91PS_AIC     pAic;

  /* Load System pAic Base address */
  pAic = AT91C_BASE_AIC;

  /* Enable User Reset and set its minimal assertion to 960 us */
  AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (0x4<<8) | (unsigned int)(0xA5<<24);

  /* First, enable the clock of the PIO */
  AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_PIOA );

  /* configure the AIC for the PIO interrupt */
  AT91F_AIC_ConfigureIt (pAic,
                         AT91C_ID_PIOA,
                         PIO_INTERRUPT_LEVEL,
                         AT91C_AIC_SRCTYPE_HIGH_LEVEL,
                         AT91F_Pio_C_Irq_Handler);

  /******************************************************************/
  /* Initialization of PIO in input mode and Input change level IRQ */
  /* - (PA14) SW4 push button                                       */
  /* - (PA15) SW3 push button                                       */
  /* - (PA2)  SQW RTC's output ( 1 HZ signal)                       */
  /******************************************************************/
  AT91F_PIO_CfgInput(AT91C_BASE_PIOA, AT91C_PIO_PA2|SW3|SW4);

  /* Enable the IRQ for the PIO in the AIC */
  AT91F_AIC_EnableIt (pAic, AT91C_ID_PIOA);

  /* Enable the IRQ on SW3, SW4 and PA2 pin at PIO level */
  AT91F_PIO_InterruptEnable(AT91C_BASE_PIOA, AT91C_PIO_PA2|SW3|SW4);

  /*** TWI intialization @ TWI_BUS_CLOCK for all TWI devices connected ***/
  AT91F_TWI_Open(TWI_BUS_CLOCK);

  /* Clear the LCD */
  AT91F_Glcd_ClearLcd();

  LcdPage = 0 ;
  /*** Probe the TWI bus to see if TWI or I2C devices are connected */
  for (i=0; i<127;i++)
  {
    I2cDevice = AT91F_TWI_ProbeDevices(AT91C_BASE_TWI, i<<16);
    if ( (I2cDevice & AT91C_TWI_NACK) != AT91C_TWI_NACK )
    {
      sprintf((char*)message,"Device @ 0x%X found", I2cDevice);
      AT91F_Glcd_PutString((char*)message,LcdPage++,0);
      AT91F_WaitSecond(1);
    }
  }

  AT91F_WaitSecond(2);

  /* Set SQW output of the RTC to generate 1HZ signal to have */
  /* a time bease for registers reading */
  AT91F_DS1337_SetControlRegister(DS1337_SQW_1HZ) ;

  /* Clear the LCD */
  AT91F_Glcd_ClearLcd();


  /* Now read the image descriptor in page 0 of the EEPROM */
  /* Write all the image and anime descriptor into page 0 */
  AT91F_Glcd_PutString("   Reading Image",LCD_PAGE_0,0);
  AT91F_Glcd_PutString("    Descriptors",LCD_PAGE_1,0);
  AT91F_Glcd_PutString(" from the EEPROM",LCD_PAGE_2,0);

  /* Read page 0 to see if the EEPROM has valid images */
  StaticImagesNum = AT91F_AT24C_ReadByte (&ptAt24c1024,0);
  AnimeImagesNum =  AT91F_AT24C_ReadByte (&ptAt24c1024,1);

  AT91F_WaitSecond(DISPLAY_DELAY);

  /* Clear the LCD */
  AT91F_Glcd_ClearLcd();

  /* Detect if valid static images exist */
  if (StaticImagesNum != 0xFF)
  {
    ValidStaticImg = 1;
    sprintf((char*)message,"%d Static Img found", StaticImagesNum);
    AT91F_Glcd_PutString((char*)message,LCD_PAGE_0,0);
    AT91F_WaitSecond(DISPLAY_DELAY-1);
  }
  else {
    StaticImagesNum = 0;
    AT91F_Glcd_PutString(" No Static Images",LCD_PAGE_0,0);
    AT91F_Glcd_PutString(" Found",LCD_PAGE_1,0);
    AT91F_WaitSecond(DISPLAY_DELAY-1);
  }

  /* Clear the LCD */
  AT91F_Glcd_ClearLcd();

  /* Detect if valid animated images exist */
  if (AnimeImagesNum != 0xFF)
  {
    ValidAnimeImg = 1;
    sprintf((char*)message," %d Animated Img", AnimeImagesNum);
    AT91F_Glcd_PutString((char*)message,LCD_PAGE_0,0);
    AT91F_Glcd_PutString(" found",LCD_PAGE_1,0);
    AT91F_WaitSecond(DISPLAY_DELAY-1);
  }
  else {
    AnimeImagesNum = 0;
    AT91F_Glcd_PutString("No Animated Images",LCD_PAGE_0,0);
    AT91F_Glcd_PutString("Found",LCD_PAGE_1,1);
    AT91F_WaitSecond(DISPLAY_DELAY-1);
  }

  AT91F_WaitSecond(DISPLAY_DELAY);

  /* If Valid static images exist, then read the descriptor */
  if (ValidStaticImg)
  {
    for (ImgIndex = 0, ByteOffSet=2 ;
         ImgIndex < StaticImagesNum ;
         ImgIndex++)
    {
      ImageTable[ImgIndex].ImgNumber = AT91F_AT24C_ReadByte (&ptAt24c1024,ByteOffSet++);
      ImageTable[ImgIndex].ImgNumber = 1;
      ImageTable[ImgIndex].ImgVsize = AT91F_AT24C_ReadByte (&ptAt24c1024,ByteOffSet++);
      ImageTable[ImgIndex].ImgHsize = AT91F_AT24C_ReadByte (&ptAt24c1024,ByteOffSet++);
      ImageTable[ImgIndex].ImgPageStart = AT91F_AT24C_ReadByte (&ptAt24c1024,ByteOffSet++);
    }
  }

  /* If Valid animated images exist, then read the descriptor */
  if (ValidAnimeImg)
  {
    for (AnimeImgIndex = 0, ByteOffSet=128 ;
         AnimeImgIndex < AnimeImagesNum;
         AnimeImgIndex++)
    {
      AnimeImgTable[AnimeImgIndex].AnimeImgNumber = AT91F_AT24C_ReadByte (&ptAt24c1024,ByteOffSet++);
      AnimeImgTable[AnimeImgIndex].AnimeImgVsize = AT91F_AT24C_ReadByte (&ptAt24c1024,ByteOffSet++);
      AnimeImgTable[AnimeImgIndex].AnimeImgHsize = AT91F_AT24C_ReadByte (&ptAt24c1024,ByteOffSet++);
      AnimeImgTable[AnimeImgIndex].AnimeImgNumOfImage = AT91F_AT24C_ReadByte (&ptAt24c1024,ByteOffSet++);
      AnimeImgTable[AnimeImgIndex].AnimeImgDelay = AT91F_AT24C_ReadByte (&ptAt24c1024,ByteOffSet++);
      AnimeImgTable[AnimeImgIndex].AnimeImgPageStart = AT91F_AT24C_ReadByte (&ptAt24c1024,ByteOffSet++);
    }
  }

  /* Get Time and Calendar information from the system time at build time */
  ptDS1337.seconds = SystemTime->tm_sec;          /* seconds */
  ptDS1337.minutes = SystemTime->tm_min;          /* minnutes */

  /* force clock to 24 hour mode. Hours GMT Time. */
  ptDS1337.hours = ((SystemTime->tm_hour) & 0x3F)+LOCAL_TIME_OFFSET;

  /* The day of week (wday) form the system time is 0 to 6.
  The RTC day of week goes from 1 to 7, so add 1 to wday */
  ptDS1337.day = SystemTime->tm_wday+1;

  ptDS1337.date = SystemTime->tm_mday;            /* day of month 1 to 31 */

  /* The month form the system time is 0 to 11. The RTC month goes from 0 to 12,
  so add 1 to the month */
  ptDS1337.month = (SystemTime->tm_mon)+1;

  /* year + Century. Assume 20YY not 19YY, and ignore DS1337_BIT_CENTURY */
  ptDS1337.year = ((SystemTime->tm_year) -100);

  /* Set Time and Calendar into the RTC */
   AT91F_DS1337_SetDateTime(&ptDS1337);

  /* Clear the LCD */
  AT91F_Glcd_ClearLcd();

  /* Call the Demos. IndexPtFunction is changed by IRQ on SW4 */
  while (1)
    ptFunctionTable[IndexPtFunction]();

}/* End */
