/*-----------------------------------------------------------------------------
*         ATMEL Microcontroller Software Support  -  ROUSSET  -
*------------------------------------------------------------------------------
* The software is delivered "AS IS" without warranty or condition of any
* kind, either express, implied or statutory. This includes without
* limitation any warranty or condition with respect to merchantability or
* fitness for any particular purpose, or against the infringements of
* intellectual property rights of others.
*------------------------------------------------------------------------------
* File Name           : Demo4All-in-one-Demo.c
* Object              : main application written in C.
*                     : Demo functions for the main App. (All-in-one-Demo.c)
*
* Version | mm | dd | yy | author :
*  1.0      01   08   07    PFi   : Creation
*
*------------------------------------------------------------------------------
* 1)Description:
*
* This file contains the different demo function to be called by the main
* application.
* It contains 3 demos:
* - AT91F_Demo1 : Simple ASCII character set and ATMEL logo display.
* - AT91F_Demo2 : Animated running cat.
* - AT91F_Demo3 : Animated Dancing pingouin.
* - AT91F_Demo4 : Calendar, Time and Temperature Demo.
*
-----------------------------------------------------------------------------*/

/* Include Standard LIB  files */
#include "Board.h"
#include "common\lib_twi.h"
#include "time.h"
#include "rtc_ds1337\ds1337.h"
#include "atmel_at24c\at24c.h"
#include "glcd_batron\glcd_i2c_batron.h"
#include "lm75\lm75.h"
#include "math.h"


/*---------------------------  Constants definition -------------------------*/
/* value of the PIT value to have 350ms @ 48 MHz
FFFFF -1 since real time out = PIV + 1 */
#define PIT_PIV_SECOND_VALUE       0xFFFFE

/* value of the PIT value to have 333ns @ 48 MHz
668 -1 since real time out = PIV + 1 */
#define PIT_PIV_MILI_SECOND_VALUE  0x667

/* value of the PIT value to have 333ns @ 48 MHz
3 -1 since real time out = PIV + 1 */
#define PIT_PIV_MICRO_SECOND_VALUE  0x2

#define DISPLAY_DELAY   3 /* in sec. Delay between displaying messages on LCD */
#define LOCAL_TIME_OFFSET   1   /* OffSet from GMT Time */

extern volatile char message[19];      /* Buffer for sprintf */
extern struct at24c ptAt24c1024 ;
extern struct tm *SystemTime;

extern volatile int ReadRtc ;

/* Global varialbes and structure definition */
extern volatile unsigned char AnimeImagesNum ;
extern volatile unsigned char StaticImagesNum ;

/* Image descriptor that will be stored in page 0 of the EEPROM */
struct Image
{
    unsigned char ImgNumber;
    unsigned char ImgVsize;
    unsigned char ImgHsize;
    unsigned char ImgPageStart;
    const unsigned char *Image;
    };

/* Declare an Image Table */
extern struct Image ImageTable[];

/* Animated bitmap descriptor */
struct AnimeImg {
    unsigned char AnimeImgNumber;
    unsigned char AnimeImgVsize;
    unsigned char AnimeImgHsize;
    unsigned char AnimeImgNumOfImage;
    unsigned char AnimeImgDelay;        /* delay between images in ms */
    unsigned char AnimeImgPageStart;
    unsigned char *Image[480];
   };

/* Declare an Image Table */
extern struct AnimeImg AnimeImgTable[];

/*-----------------------------------------------------------------------------
* Wait function with the Periodic Interval Timer (PIT)
* The wait time unit is the max PIT rollover value, i.e 350ms @ 48 MHz
*----------------------------------------------------------------------------*/
void AT91F_WaitSecond (unsigned int seconde)
{
   unsigned int PitStatus = 0;  /* Status register of the PIT */
   unsigned int PitLoop = 0;    /* Store the number of PIT Loop */

    AT91C_BASE_PITC->PITC_PIMR = AT91C_PITC_PITEN|PIT_PIV_SECOND_VALUE;

   for( PitLoop=0; PitLoop <(seconde*3);)    /* One PIT loop equals 333ms */
   {
    /* Wait for the PIT counter overflow occurs */
     while ((AT91C_BASE_PITC->PITC_PISR & AT91C_PITC_PITS)==0);
    /* Read the PIT Interval Value Reg to clear it for the next overflow */
    PitStatus = AT91C_BASE_PITC->PITC_PIVR ;
    /* dummy access to avoid IAR warning */
    PitStatus = PitStatus ;
    PitLoop++;
   }
}

/*-----------------------------------------------------------------------------
/ Wait function with the Periodic Interval Timer (PIT)
/ The wait time is from 1ms to 999ms.
/----------------------------------------------------------------------------*/
void AT91F_WaitMiliSecond (unsigned int MiliSeconds)
{
   unsigned int PitStatus = 0;     /* Status register of the PIT */
   unsigned int PitLoop = 0;    /* Store the number of PIT Loop */

   AT91C_BASE_PITC->PITC_PIMR = AT91C_PITC_PITEN|PIT_PIV_MILI_SECOND_VALUE;

   for( PitLoop=0; PitLoop <(MiliSeconds*3);)   /* One PIT loop equals 333ms */
   {
    /* Wait for the PIT counter overflow occurs */
    while ((AT91C_BASE_PITC->PITC_PISR & AT91C_PITC_PITS)==0);
    /* Read the PIT Interval Value Reg. to clear it for the next overflow */
    PitStatus = AT91C_BASE_PITC->PITC_PIVR ;
    /* dummy access to avoid IAR warning */
    PitStatus = PitStatus ;
    PitLoop++;
   }
}


/*-----------------------------------------------------------------------------
* Wait function with the Periodic Interval Timer (PIT)
* The wait time is from 1us to 999us.
-----------------------------------------------------------------------------*/
void AT91F_Wait_MicroSecond (unsigned int MicroSecond)
{
   unsigned int PitStatus = 0;     /* Status register of the PIT */
   unsigned int PitLoop = 0;    /* Store the number of PIT Loop */

   AT91C_BASE_PITC->PITC_PIMR = AT91C_PITC_PITEN|PIT_PIV_MICRO_SECOND_VALUE;

   for( PitLoop=0; PitLoop <(MicroSecond);)   /* One PIT loop equals 333ns */
   {
    /* Wait for the PIT counter overflow occurs */
    while ((AT91C_BASE_PITC->PITC_PISR & AT91C_PITC_PITS)==0);
    /* Read the PIT Interval Value Reg. to clear it for the next overflow */
    PitStatus = AT91C_BASE_PITC->PITC_PIVR ;
    /* dummy access to avoid IAR warning */
    PitStatus = PitStatus ;
    PitLoop++;
   }
}

/* -----------------------------------------------------------------*/
/* AT91F_Demo1 : Simple ASCII character set and ATMEL logo display. */
/* -----------------------------------------------------------------*/
void AT91F_Demo1(void)
{

    unsigned int PageAddress;

  /* GLCD and images variables */
  char TempImgBuffer[(96*40)/8];
  unsigned int ImgIndex;

  /* Clear the LCD */
  AT91F_Glcd_ClearLcd();
  /* Printing the Standard ASCII CHARACTER SET 33 to 126 */
  AT91F_Glcd_PutString("     Printing",LCD_PAGE_0,0);
  AT91F_Glcd_PutString("        the",LCD_PAGE_1,0);
  AT91F_Glcd_PutString("   Standard ASCII",LCD_PAGE_2,0);
  AT91F_Glcd_PutString("   Character Set",LCD_PAGE_3,0);
  AT91F_WaitSecond(2);

    /* Clear the LCD and put it in normal mode */
    AT91F_Glcd_ClearLcd();
    AT91F_Glcd_VideoMode(Normal);

    AT91F_WaitSecond(1);

    /* Printing on page 0.  /!\ To print the " character, use (\") */
    AT91F_Glcd_PutString(" !\"#$%&'()*+,-./012",LCD_PAGE_0,0);
    /* Printing on page 1 */
    AT91F_Glcd_PutString("3456789:;<=>?@ABCDE",LCD_PAGE_1,0);
    /* Printing on page 2 */
    AT91F_Glcd_PutString("FGHIJKLMNOPQRSTUVWX",LCD_PAGE_2,0);
    /* Printing on page 3 */
    AT91F_Glcd_PutString("YZ[\"]^_`abcdefghijk",LCD_PAGE_3,0);
    /* Printing on page 4 */
    AT91F_Glcd_PutString("lmnopqrstuvwxyz{|}~",LCD_PAGE_4,0);

    AT91F_WaitSecond(1);

    /* Clear the LCD and put it in normal mode */
    AT91F_Glcd_ClearLcd();
    AT91F_Glcd_VideoMode(Normal);

    /* Printing the Extended ASCII CHARACTER SET */
    AT91F_Glcd_PutString("     Printing",LCD_PAGE_0,0);
    AT91F_Glcd_PutString("        the",LCD_PAGE_1,0);
    AT91F_Glcd_PutString("  Extended ASCII",LCD_PAGE_2,0);
    AT91F_Glcd_PutString("  Character Set",LCD_PAGE_3,0);

    AT91F_WaitSecond(1);

    /* Clear the LCD and put it in normal mode */
    AT91F_Glcd_ClearLcd();
    AT91F_Glcd_VideoMode(Normal);

    /* Printing on page 0 */
    AT91F_Glcd_PutString("",LCD_PAGE_0,0);
    /* Printing on page 1 */
    AT91F_Glcd_PutString("",LCD_PAGE_1,0);
    /* Printing on page 2 */
    AT91F_Glcd_PutString("",LCD_PAGE_2,0);
    /* Printing on page 3 */
    AT91F_Glcd_PutString("",LCD_PAGE_3,0);
    /* Printing on page 4 */
    AT91F_Glcd_PutString("",LCD_PAGE_4,0);

    AT91F_WaitSecond(1);

    /* Clear the LCD and put it in normal mode */
    AT91F_Glcd_ClearLcd();

  /******************************************/
  /* Read the static images from the EEPROM */
  /******************************************/
  for (ImgIndex = 0 ; ImgIndex < StaticImagesNum ; ImgIndex++)
  {
    PageAddress = ImageTable[ImgIndex].ImgPageStart * ptAt24c1024.PageSize ;

    /* Read the image stored in the EEPROM */
    AT91F_AT24C_SequentialRead (&ptAt24c1024, TempImgBuffer, sizeof TempImgBuffer, PageAddress);
    AT91F_Glcd_DisplayBitmap((unsigned char*)TempImgBuffer,0,0,
                             ImageTable[ImgIndex].ImgHsize,
                             ImageTable[ImgIndex].ImgVsize);
    AT91F_WaitSecond(DISPLAY_DELAY-1);
    AT91F_Glcd_VideoMode(Inverse);
    AT91F_WaitSecond(DISPLAY_DELAY-1);
    AT91F_Glcd_ClearLcd();
  }

}

/* ------------------------------*/
/* AT91F_Demo2 : Animated images */
/* ------------------------------*/
void AT91F_Demo2 (void)
{
  /* TWI AT24C1024 EEPROM variables */
  unsigned int PageAddress;

  /* GLCD and images variables */
  char TempImgBuffer[(96*40)/8];
  unsigned int AnimeImgIndex=1;
  unsigned int AnimeSubImgIndex;

  /* Clear the LCD */
  AT91F_Glcd_ClearLcd();

  /********************************************/
  /* Read the Animated images from the EEPROM */
  /********************************************/
  PageAddress = ptAt24c1024.PageSize * AnimeImgTable[AnimeImgIndex].AnimeImgPageStart ;
  for (AnimeSubImgIndex=0;
       AnimeSubImgIndex < AnimeImgTable[AnimeImgIndex].AnimeImgNumOfImage;
       AnimeSubImgIndex++)
  {
    /* Read the image stored in the EEPROM */
    AT91F_AT24C_SequentialRead (&ptAt24c1024,
                                TempImgBuffer,
                                (AnimeImgTable[AnimeImgIndex].AnimeImgHsize*AnimeImgTable[AnimeImgIndex].AnimeImgVsize)/8,
                                PageAddress);
    AT91F_Glcd_ClearLcd();
    AT91F_Glcd_DisplayBitmap((unsigned char*)TempImgBuffer,0,0,
                             AnimeImgTable[AnimeImgIndex].AnimeImgHsize,
                             AnimeImgTable[AnimeImgIndex].AnimeImgVsize);
                             PageAddress = PageAddress + 2*256 ;  /* Each image takes 480 bytes max */
    AT91F_WaitMiliSecond(AnimeImgTable[AnimeImgIndex].AnimeImgDelay);
  }
}
/*-------------------------------------*/
/* AT91F_Demo3 : Animated running cat. */
/*-------------------------------------*/
void AT91F_Demo3(void)
{
 /* TWI AT24C1024 EEPROM variables */
  unsigned int PageAddress;

  /* GLCD and images variables */
  char TempImgBuffer[(96*40)/8];
  unsigned int AnimeImgIndex=0;
  unsigned int AnimeSubImgIndex;

  /* Clear the LCD */
  AT91F_Glcd_ClearLcd();

  /********************************************/
  /* Read the Animated images from the EEPROM */
  /********************************************/
  PageAddress = ptAt24c1024.PageSize * AnimeImgTable[AnimeImgIndex].AnimeImgPageStart ;
  for (AnimeSubImgIndex=0;
       AnimeSubImgIndex < AnimeImgTable[AnimeImgIndex].AnimeImgNumOfImage;
       AnimeSubImgIndex++)
  {
    /* Read the image stored in the EEPROM */
    AT91F_AT24C_SequentialRead (&ptAt24c1024,
                                TempImgBuffer,
                                (AnimeImgTable[AnimeImgIndex].AnimeImgHsize*AnimeImgTable[AnimeImgIndex].AnimeImgVsize)/8,
                                PageAddress);
    AT91F_Glcd_ClearLcd();
    AT91F_Glcd_DisplayBitmap((unsigned char*)TempImgBuffer,0,0,
                             AnimeImgTable[AnimeImgIndex].AnimeImgHsize,
                             AnimeImgTable[AnimeImgIndex].AnimeImgVsize);
                             PageAddress = PageAddress + 2*256 ;  /* Each image takes 480 bytes max */
    AT91F_WaitMiliSecond(AnimeImgTable[AnimeImgIndex].AnimeImgDelay);
  }

}

/*----------------------------------------------------*/
/* AT91F_Demo4 : Calendar, Time and Temperature Demo. */
/*----------------------------------------------------*/
void AT91F_Demo4(void)
{
  /* LM75 Temp Sensor variables */
  float TempCelsius ;       /* Temperature in degree Celsius */
  float TempFahrenheit ;   /* Temperature in degree Fahrenhheit */

  /* RTC DS1337 variables */
  struct  DS1337 ptDS1337 ;
  unsigned char GmtHours;

  /* Date, Time and Temperature reading is every 1 seconds */
  if (ReadRtc)
  {
    /* Clear the LCD */
    AT91F_Glcd_ClearLcdPage(LCD_PAGE_1);

    ReadRtc = 0 ;
    TempCelsius = AT91F_LM75_ReadTempRegister();

    /* To convert Celsius to Fahrenheit, multiply by 1.8 and add 32 degrees */
    TempFahrenheit = (TempCelsius * 1.8) + 32 ;

    sprintf((char*) message,
            "T= %3.1fC-%3.1fF",
            (float)TempCelsius,
            (float)TempFahrenheit);

    AT91F_Glcd_PutString((char*)message,LCD_PAGE_0,0);

    /* Get the Time and Date from the RTC */
    AT91F_DS1337_ReadDateTime(&ptDS1337);

    /* Print date on the LCD */
    AT91F_Glcd_ClearLcdPage(LCD_PAGE_2);

    sprintf((char*)message,
            "%s %d %s 20%02d",
            DayOfWeek[(ptDS1337.day)-1],
            ptDS1337.date,
            Month[(ptDS1337.month)-1],
            ptDS1337.year);

    AT91F_Glcd_PutString((char*)message,LCD_PAGE_2,0);

    if ((ptDS1337.hours)==0)
      GmtHours = 23;
    else GmtHours = (ptDS1337.hours)-LOCAL_TIME_OFFSET;

    /* Print time on the LCD. GMT and Local time */
    AT91F_Glcd_ClearLcdPage(LCD_PAGE_3);
    sprintf((char*)message,
            "GMT: %02d : %02d : %02d",
            GmtHours,
            ptDS1337.minutes,
            ptDS1337.seconds);

    AT91F_Glcd_PutString((char*)message,LCD_PAGE_3,0);
    AT91F_Glcd_ClearLcdPage(LCD_PAGE_4);
    sprintf((char*)message,
            "Local: %02d : %02d : %02d",
            ptDS1337.hours,
            ptDS1337.minutes,
            ptDS1337.seconds);

    AT91F_Glcd_PutString((char*)message,LCD_PAGE_4,0);

    }
}

