/*-----------------------------------------------------------------------------
*         ATMEL Microcontroller Software Support  -  ROUSSET  -
*------------------------------------------------------------------------------
* The software is delivered "AS IS" without warranty or condition of any
* kind, either express, implied or statutory. This includes without
* limitation any warranty or condition with respect to merchantability or
* fitness for any particular purpose, or against the infringements of
* intellectual property rights of others.
*------------------------------------------------------------------------------
* File Name           : EEPROM-Image-Loader.c
* Object              : main application written in C.
*
*                     : The Following code is for the following devices:
*                       - AT24C128/C256/C512/C1024
*
* Version | mm | dd | yy | author :
*  1.0      12   17   06    PFi   : Creation
*
*------------------------------------------------------------------------------
* 1)Description:
*
* ATMEL Two-wire Serial EEPROM AT24C test.
*
* ------| AT91 / IO  | AT24C / IO
* CLOCK | TWCK / PA3 | SCL / 6
* DATA  | TWD / PA4  | SDA / 5
*

* * WARNINGS ** :
* - No pull ups are mounted on the board on TWD and TWCK. 2.2K Pull up used
* - By default IAR compiler use the Tiny printf and scanf library format.
*   To be able to make print of float variables, you must set the library
*   format to large.
*
*  For further details check the following Application note
*  "AT91-AN01: Using the Two-wire interface (TWI) in
*  Master Mode on AT91SAM Microcontrollers."
*  www.atmel.com/dyn/products/app_notes.asp?family_id=605
*
* - By default IAR compiler use the Tiny printf and scanf library format.
*   To be able to make print of float variables, you must set the library
*   format to large.
*
* This example runs in polling mode.and makes no use of IRQs. To have further
* details and implementation example about using IRQs, refer to any Interrupt
* project examples provided in Getting Started available on
* www.atmel.com/dyn/products/app_notes.asp?family_id=605*
******************************************************************************/
/* Include Standard LIB  files */
#include "..\compil\srciar\board.h"
#include "atmel_at24c\at24c.h"
#include "common\init.h"
#include "common\lib_twi.h"
#include "glcd_batron\glcd_i2c_batron.h"
//#include "glcd_batron\font\full_5_x_7_charset.h"

/* Include the images that must be downloded here */
#include "glcd_batron\Images\atmel.h"
#include "glcd_batron\Images\anime_cat.h"
#include "glcd_batron\Images\anime_pingouin.h"


/* Constant and macro definition */
/* value of the PIT value to have 350ms @ 48 MHz
FFFFF -1 since real time out = PIV + 1 */
#define PIT_PIV_SECOND_VALUE       0xFFFFE

/* value of the PIT value to have 333ns @ 48 MHz
668 -1 since real time out = PIV + 1 */
#define PIT_PIV_MILI_SECOND_VALUE  0x667

/* value of the PIT value to have 333ns @ 48 MHz
3 -1 since real time out = PIV + 1 */
#define PIT_PIV_MICRO_SECOND_VALUE  0x2

#define TWI_BUS_CLOCK   400000 /* in Hz*/

#define DISPLAY_DELAY   3 /* in sec. Delay between displaying messages on LCD */

/*
Each image can have a size of 480 bytes (96x40)/8 which is needs 2 pages of
the EERPOM. The maximum number of images in the AT24C1024 EERPOM will be
511/2 = 255.
Page 0 of the EERPOM (256 bytes) contains the image descriptor. the first
128th bytes are for static images, and the rest for Animated Images
descriptors.
*/
#define MAX_NUM_IMAGE   255     /* Define the maximum number of images here */
#define NUM_STATIC_IMAGE   1    /* Define the number of static images here */

#define NUM_ANIME_IMAGE    2    /* Define the number of Animeted images here */
#define MAX_IMAGE_PER_ANIME   12 /* Number of sub-images of animated images */

/* Global variables and structure definition */
/* Image descriptor that will be stored in page 0 of the EEPROM */
struct Image {
    unsigned char ImgNumber;
    unsigned char ImgVsize;
    unsigned char ImgHsize;
    unsigned char ImgPageStart;
    const unsigned char *Image;
    };

/* Declare an Image Table */
struct Image ImageTable[NUM_STATIC_IMAGE];

/* Animated bitmap descriptor */
struct AnimeImg {
    unsigned char AnimeImgNumber;
    unsigned char AnimeImgVsize;
    unsigned char AnimeImgHsize;
    unsigned char AnimeImgNumOfImage;
    unsigned char AnimeImgDelay;        /* delay between images in ms */
    unsigned char AnimeImgPageStart;
    unsigned char *Image[480];
    };

/* Declare an Image Table */
struct AnimeImg AnimeImgTable[NUM_ANIME_IMAGE];

volatile char message[19];      /* Buffer for sprintf */

/* External function prototype */

/*-----------------------------------------------------------------------------
/ Wait function with the Periodic Interval Timer (PIT)
/ The wait time unit is the max PIT rollover value, i.e 350ms @ 48 MHz
/----------------------------------------------------------------------------*/
void AT91F_WaitSecond (unsigned int seconde)
{
   unsigned int PitStatus = 0;  /* Status register of the PIT */
   unsigned int PitLoop = 0;    /* Store the number of PIT Loop */

   AT91C_BASE_PITC->PITC_PIMR = AT91C_PITC_PITEN|PIT_PIV_SECOND_VALUE;

   for( PitLoop=0; PitLoop <(seconde*3);)    /* One PIT loop equals 333ms */
   {
    /* Wait for the PIT counter overflow occurs */
    while ((AT91C_BASE_PITC->PITC_PISR & AT91C_PITC_PITS)==0);
    /* Read the PIT Interval Value Reg to clear it for the next overflow */
    PitStatus = AT91C_BASE_PITC->PITC_PIVR ;
    /* dummy access to avoid IAR warning */
    PitStatus = PitStatus ;
    PitLoop++;
   }
}

/*-----------------------------------------------------------------------------
* Wait function with the Periodic Interval Timer (PIT)
* The wait time is from 1ms to 999ms.
-----------------------------------------------------------------------------*/
void AT91F_WaitMiliSecond (unsigned int MiliSeconds)
{
   unsigned int PitStatus = 0;     /* Status register of the PIT */
   unsigned int PitLoop = 0;    /* Store the number of PIT Loop */

   AT91C_BASE_PITC->PITC_PIMR = AT91C_PITC_PITEN|PIT_PIV_MILI_SECOND_VALUE;

   for( PitLoop=0; PitLoop <(MiliSeconds*3);)   /* One PIT loop equals 333ms */
   {
    /* Wait for the PIT counter overflow occurs */
    while ((AT91C_BASE_PITC->PITC_PISR & AT91C_PITC_PITS)==0);
    /* Read the PIT Interval Value Reg. to clear it for the next overflow */
    PitStatus = AT91C_BASE_PITC->PITC_PIVR ;
    /* dummy access to avoid IAR warning */
    PitStatus = PitStatus ;
    PitLoop++;
   }
}

/*-----------------------------------------------------------------------------
* Wait function with the Periodic Interval Timer (PIT)
* The wait time is from 1us to 999us.
-----------------------------------------------------------------------------*/
void AT91F_WaitMicroSecond (unsigned int MicroSecond)
{
   unsigned int PitStatus = 0;     /* Status register of the PIT */
   unsigned int PitLoop = 0;    /* Store the number of PIT Loop */

   AT91C_BASE_PITC->PITC_PIMR = AT91C_PITC_PITEN|PIT_PIV_MICRO_SECOND_VALUE;

   for( PitLoop=0; PitLoop < (MicroSecond*3);)   /* One PIT loop equals 333ns */
   {
    /* Wait for the PIT counter overflow occurs */
    while ((AT91C_BASE_PITC->PITC_PISR & AT91C_PITC_PITS)==0);
    /* Read the PIT Interval Value Reg. to clear it for the next overflow */
    PitStatus = AT91C_BASE_PITC->PITC_PIVR ;
    /* dummy access to avoid IAR warning */
    PitStatus = PitStatus ;
    PitLoop++;
   }
}

/*-----------------------------------------------------------------------------
* Function Name       : main
* Object              : Main function
* Input Parameters    : none
* Output Parameters   : TRUE
-----------------------------------------------------------------------------*/
int main( void )
{
    struct at24c ptAt24c1024 ;
    char TxBuffer[256] ;
    char RxBuffer[256] ;
    char TempImgBuffer[(96*40)/8];
    int BufferIndex;
    int Index;
    unsigned int PageCount;
    unsigned int PageAddress;
    unsigned int error = 0;
    unsigned int ImgIndex;
    unsigned int ImgSize;
    unsigned int AnimeImgIndex;
    unsigned int AnimeSubImgIndex;
    unsigned int AnimeImgSize;
    unsigned char *ImgPt;


    /* Init for atmel_logo.h */
    ImageTable[0].ImgNumber = 1;
    ImageTable[0].ImgVsize = 40;
    ImageTable[0].ImgHsize = 96;
    /* ImgPageStart is not computed automaticaly versus the number
     of static and animated images. This should be set by the user.
     Add the page start of the first or previous static animated images plus
     its size */
    ImageTable[0].ImgPageStart = 1;
    ImageTable[0].Image = AtmelLogo;

    /* Pingouin Anime */
    AnimeImgTable[0].AnimeImgNumber = 1 ;
    AnimeImgTable[0].AnimeImgVsize = 40 ;
    AnimeImgTable[0].AnimeImgHsize = 96 ;
    AnimeImgTable[0].AnimeImgNumOfImage = 12 ;
    AnimeImgTable[0].AnimeImgDelay = 120 ;

    /* ImgPageStart is not computed automaticaly versus the number
     of static and animated images. This should be set by the user.
     Add the page start of the first or previous static animated images plus
     its size */
    AnimeImgTable[0].AnimeImgPageStart = 3 ;
    AnimeImgTable[0].Image[0] = Pingouin1 ;
    AnimeImgTable[0].Image[1] = Pingouin2 ;
    AnimeImgTable[0].Image[2] = Pingouin3 ;
    AnimeImgTable[0].Image[3] = Pingouin4 ;
    AnimeImgTable[0].Image[4] = Pingouin5 ;
    AnimeImgTable[0].Image[5] = Pingouin6 ;
    AnimeImgTable[0].Image[6] = Pingouin7 ;
    AnimeImgTable[0].Image[7] = Pingouin8 ;
    AnimeImgTable[0].Image[8] = Pingouin9 ;
    AnimeImgTable[0].Image[9] = Pingouin10 ;
    AnimeImgTable[0].Image[10] = Pingouin11 ;
    AnimeImgTable[0].Image[11] = Pingouin12 ;

    /* Running cat anime */
    AnimeImgTable[1].AnimeImgNumber = 2 ;
    AnimeImgTable[1].AnimeImgVsize = 32 ;
    AnimeImgTable[1].AnimeImgHsize = 72 ;
    AnimeImgTable[1].AnimeImgNumOfImage = 7 ;
    AnimeImgTable[1].AnimeImgDelay = 100 ;
    /* ImgPageStart is not computed automaticaly versus the number
     of static and animated images. This should be set by the user.
     Add the page start of the first or previous static animated images plus
     its size */
    AnimeImgTable[1].AnimeImgPageStart = 27 ;
    AnimeImgTable[1].Image[0] = cat1 ;
    AnimeImgTable[1].Image[1] = cat2 ;
    AnimeImgTable[1].Image[2] = cat3 ;
    AnimeImgTable[1].Image[3] = cat4 ;
    AnimeImgTable[1].Image[4] = cat5 ;
    AnimeImgTable[1].Image[5] = cat6 ;
    AnimeImgTable[1].Image[6] = cat7 ;

    ptAt24c1024.PageSize = 256;
    ptAt24c1024.NumOfPage = 512;
    ptAt24c1024.EepromSize = 256*512;
    ptAt24c1024.SlaveAddress = AT24C_I2C_ADDRESS;
    ptAt24c1024.EepromName = AT24C1024;

    /* Make a bus recovery sequence in case of the transmission has been
    broken due to a power loss or reset */
    AT91F_AT24C_MemoryReset (AT91C_BASE_PIOA,
                             AT91C_ID_PIOA,
                             AT91C_PA3_TWD,
                             AT91C_PA4_TWCK);

    /* TWI intialization @ TWI_BUS_CLOCK for all TWI devices
       connected on the bus */
    AT91F_TWI_Open(TWI_BUS_CLOCK);


    /* Clear the LCD */
    AT91F_Glcd_ClearLcd();

    /* Write all the image and anime descriptor into page 0 */
    /* Static images from 0 to 127 and Anime images from 128 to 255 */

    /***********************************************************************/
    /*       Before writing descriptors fill the EEPROM with 0xFF          */
    /***********************************************************************/
    AT91F_Glcd_PutString("    Initializing   ",LCD_PAGE_0,0);
    AT91F_Glcd_PutString("the EEPROM to 0xFF ",LCD_PAGE_1,0);
    AT91F_Glcd_PutString("   and verifying   ",LCD_PAGE_2,0);

    AT91F_WaitSecond(DISPLAY_DELAY);

    /* Initialize the Transmit buffer */
    for (BufferIndex=0; BufferIndex < sizeof(TxBuffer); BufferIndex++)
    {
      TxBuffer[BufferIndex] =0xFF ;
    }

    /* Clear the Receive buffer */
    for (BufferIndex=0; BufferIndex < sizeof(RxBuffer); BufferIndex++)
    {
      RxBuffer[BufferIndex] = 0 ;
    }

    /* Write the whole EEPROM with 0xFF and verify */
    for (PageCount=0 ; PageCount < 512 ; PageCount++)
    {
      AT91F_AT24C_WritePage (&ptAt24c1024, TxBuffer, sizeof TxBuffer,PageCount,0);

      /* Read the page to check 0xFF */
      AT91F_AT24C_ReadPage (&ptAt24c1024, RxBuffer, sizeof RxBuffer,PageCount,0);
      for (BufferIndex=0; BufferIndex < sizeof(TxBuffer); BufferIndex++)
      {
        if (TxBuffer[BufferIndex] != RxBuffer[BufferIndex])
          error++;
      }

      sprintf((char*)message,"Page %d   ", PageCount);
      AT91F_Glcd_PutString((char*)message,LCD_PAGE_3,0);
    }

    if (error)
    {
        AT91F_Glcd_PutString("Error when reading",LCD_PAGE_0,0);
        AT91F_Glcd_PutString("the EEPROM",LCD_PAGE_1,0);
        AT91F_WaitSecond(DISPLAY_DELAY);
    }

    /* Clear the LCD */
    AT91F_Glcd_ClearLcd();

    /* Write all the image and anime descriptor into page 0 */
    AT91F_Glcd_PutString("   Writing Image   ",LCD_PAGE_0,0);
    AT91F_Glcd_PutString("    Descriptors    ",LCD_PAGE_1,0);
    AT91F_Glcd_PutString("  into the EEPROM  ",LCD_PAGE_2,0);

    AT91F_WaitSecond(DISPLAY_DELAY);

    /* The first parameter of the image descriptor is the number
    of images stored */
    AT91F_AT24C_WriteByte (&ptAt24c1024, NUM_STATIC_IMAGE, 0);
    AT91F_AT24C_WriteByte (&ptAt24c1024, NUM_ANIME_IMAGE, 1);


    /* Static images from byte 2 to 127 */
    AT91F_AT24C_WriteByte (&ptAt24c1024,ImageTable[0].ImgNumber, 2);
    AT91F_AT24C_WriteByte (&ptAt24c1024,ImageTable[0].ImgVsize, 3);
    AT91F_AT24C_WriteByte (&ptAt24c1024,ImageTable[0].ImgHsize, 4);
    AT91F_AT24C_WriteByte (&ptAt24c1024,ImageTable[0].ImgPageStart, 5);

    /* Anime images from byte 128 to 255 */

    /* Pingouin Anime */
    AT91F_AT24C_WriteByte (&ptAt24c1024,AnimeImgTable[0].AnimeImgNumber, 0+128);
    AT91F_AT24C_WriteByte (&ptAt24c1024,AnimeImgTable[0].AnimeImgVsize, 1+128);
    AT91F_AT24C_WriteByte (&ptAt24c1024,AnimeImgTable[0].AnimeImgHsize, 2+128);
    AT91F_AT24C_WriteByte (&ptAt24c1024,AnimeImgTable[0].AnimeImgNumOfImage, 3+128);
    AT91F_AT24C_WriteByte (&ptAt24c1024,AnimeImgTable[0].AnimeImgDelay, 4+128);
    AT91F_AT24C_WriteByte (&ptAt24c1024,AnimeImgTable[0].AnimeImgPageStart, 5+128);

    /* Running cat anime */
    AT91F_AT24C_WriteByte (&ptAt24c1024,AnimeImgTable[1].AnimeImgNumber, 6+128);
    AT91F_AT24C_WriteByte (&ptAt24c1024,AnimeImgTable[1].AnimeImgVsize, 7+128);
    AT91F_AT24C_WriteByte (&ptAt24c1024,AnimeImgTable[1].AnimeImgHsize, 8+128);
    AT91F_AT24C_WriteByte (&ptAt24c1024,AnimeImgTable[1].AnimeImgNumOfImage, 9+128);
    AT91F_AT24C_WriteByte (&ptAt24c1024,AnimeImgTable[1].AnimeImgDelay, 10+128);
    AT91F_AT24C_WriteByte (&ptAt24c1024,AnimeImgTable[1].AnimeImgPageStart, 11+128);

    /* Clear the LCD */
    AT91F_Glcd_ClearLcd();

    sprintf((char*)message,"- %d Static", NUM_STATIC_IMAGE);
    AT91F_Glcd_PutString((char*)message,LCD_PAGE_0,0);
    sprintf((char*)message,"- %d Animated", NUM_ANIME_IMAGE);
    AT91F_Glcd_PutString((char*)message,LCD_PAGE_1,0);
    AT91F_Glcd_PutString("Writen in Page 0",LCD_PAGE_2,0);

    AT91F_WaitSecond(DISPLAY_DELAY);

    /***********************************************************************/
    /*                 Write the images into the EEPORM                    */
    /***********************************************************************/
    /* Clear the LCD */
    AT91F_Glcd_ClearLcd();
    AT91F_Glcd_PutString("  Writing Static   ",LCD_PAGE_0,0);
    AT91F_Glcd_PutString("      Images       ",LCD_PAGE_1,0);
    AT91F_Glcd_PutString("  into the EEPROM  ",LCD_PAGE_2,0);

    AT91F_WaitSecond(DISPLAY_DELAY);

    /*************************/
    /* Writing Static images */
    /*************************/
    for (ImgIndex = 0 ; ImgIndex < NUM_STATIC_IMAGE ; ImgIndex++)
    {
      PageCount = ImageTable[ImgIndex].ImgPageStart ;

      for (ImgSize=0;
           ImgSize < ((ImageTable[ImgIndex].ImgHsize*ImageTable[ImgIndex].ImgVsize)/8);)
      {
        /* Read the first 256 bytes and write them into a page */
        for (Index=0 ; Index < 256 ; Index++)
        {
          TxBuffer[Index] = *(ImageTable[ImgIndex].Image++) ;
          ImgSize++ ;
        }

        /* Write a complete page */
        AT91F_AT24C_WritePage (&ptAt24c1024, TxBuffer, sizeof TxBuffer,PageCount,0);
        PageCount++;
      }
      sprintf((char*)message,"Image %d stored  ", ImgIndex);
      AT91F_Glcd_PutString((char*)message,LCD_PAGE_3,0);
      AT91F_WaitSecond(DISPLAY_DELAY);
    }

    sprintf((char*)message," %d Image(s) stored", NUM_STATIC_IMAGE);
    AT91F_Glcd_PutString((char*)message,LCD_PAGE_4,0);

    AT91F_WaitSecond(DISPLAY_DELAY);

    /* Clear the LCD */
    AT91F_Glcd_ClearLcd();


   /***************************/
   /* Writing Animated images */
   /***************************/
    /* Clear the LCD */
    AT91F_Glcd_ClearLcd();
    AT91F_Glcd_PutString("  Writing Animated   ",LCD_PAGE_0,0);
    AT91F_Glcd_PutString("      Images       ",LCD_PAGE_1,0);
    AT91F_Glcd_PutString("  into the EEPROM  ",LCD_PAGE_2,0);
    AT91F_WaitSecond(DISPLAY_DELAY);

    for (AnimeImgIndex = 0; AnimeImgIndex < NUM_ANIME_IMAGE; AnimeImgIndex++)
    {
      PageCount = AnimeImgTable[AnimeImgIndex].AnimeImgPageStart ;

      for (AnimeSubImgIndex=0;
           AnimeSubImgIndex < AnimeImgTable[AnimeImgIndex].AnimeImgNumOfImage;AnimeSubImgIndex++)
      {
        ImgPt = (unsigned char*)(AnimeImgTable[AnimeImgIndex].Image[AnimeSubImgIndex]);

        for (AnimeImgSize=0;
             AnimeImgSize < ((AnimeImgTable[AnimeImgIndex].AnimeImgHsize*AnimeImgTable[AnimeImgIndex].AnimeImgVsize)/8);)
        {
          /* Read the first 256 bytes and write them into a page */
          for (Index=0 ; Index < 256 ; Index++)
          {
            TxBuffer[Index] = *ImgPt++ ;
            AnimeImgSize++ ;
          }

          /* Write a complete page */
          AT91F_AT24C_WritePage (&ptAt24c1024, TxBuffer, sizeof TxBuffer,PageCount,0);
          PageCount++;
        }
      }
      sprintf((char*)message,"Image %d stored  ", AnimeImgIndex);
      AT91F_Glcd_PutString((char*)message,LCD_PAGE_3,0);
      AT91F_WaitSecond(DISPLAY_DELAY);
    }

    sprintf((char*)message," %d Image(s) stored", NUM_ANIME_IMAGE);
    AT91F_Glcd_PutString((char*)message,LCD_PAGE_4,0);

    AT91F_WaitSecond(DISPLAY_DELAY);

    /* Clear the LCD */
    AT91F_Glcd_ClearLcd();

    AT91F_Glcd_PutString("  Reading Image(s)",LCD_PAGE_0,0);
    AT91F_Glcd_PutString("      Images",LCD_PAGE_1,0);
    AT91F_Glcd_PutString("    from EEPROM",LCD_PAGE_2,0);

    AT91F_WaitSecond(DISPLAY_DELAY);

      /******************************************/
   /* Read the static images from the EEPROM */
   /******************************************/
    for (ImgIndex = 0 ; ImgIndex < NUM_STATIC_IMAGE ; ImgIndex++)
    {
      PageAddress = ImageTable[ImgIndex].ImgPageStart * ptAt24c1024.PageSize ;

       /* Read the image stored in the EEPROM */
       AT91F_AT24C_SequentialRead (&ptAt24c1024, TempImgBuffer, sizeof TempImgBuffer, PageAddress);

       AT91F_Glcd_DisplayBitmap((unsigned char*)TempImgBuffer,0,0,
                                ImageTable[ImgIndex].ImgHsize,
                                ImageTable[ImgIndex].ImgVsize);
       AT91F_WaitSecond(DISPLAY_DELAY);
       AT91F_Glcd_ClearLcd();
     }

   /********************************************/
   /* Read the Animated images from the EEPROM */
   /********************************************/
   while (1)
   {
   for (AnimeImgIndex = 0; AnimeImgIndex < NUM_ANIME_IMAGE; AnimeImgIndex++)
   {
     PageAddress = ptAt24c1024.PageSize * AnimeImgTable[AnimeImgIndex].AnimeImgPageStart ;
     for (AnimeSubImgIndex=0;
          AnimeSubImgIndex < AnimeImgTable[AnimeImgIndex].AnimeImgNumOfImage;
          AnimeSubImgIndex++)
     {
       /* Read the image stored in the EEPROM */
       AT91F_AT24C_SequentialRead (&ptAt24c1024,
                                   TempImgBuffer,
                                   (AnimeImgTable[AnimeImgIndex].AnimeImgHsize*AnimeImgTable[AnimeImgIndex].AnimeImgVsize)/8,
                                   PageAddress);
       AT91F_Glcd_ClearLcd();

       AT91F_Glcd_DisplayBitmap((unsigned char*)TempImgBuffer,0,0,
                                AnimeImgTable[AnimeImgIndex].AnimeImgHsize,
                                AnimeImgTable[AnimeImgIndex].AnimeImgVsize);

       PageAddress = PageAddress + 2*256 ;  /* Each image takes 480 bytes max */
       AT91F_WaitMiliSecond(AnimeImgTable[AnimeImgIndex].AnimeImgDelay);
     }
   }
   }

}

