/*-----------------------------------------------------------------------------
*         ATMEL Microcontroller Software Support  -  ROUSSET  -
*------------------------------------------------------------------------------
* The software is delivered "AS IS" without warranty or condition of any
* kind, either express, implied or statutory. This includes without
* limitation any warranty or condition with respect to merchantability or
* fitness for any particular purpose, or against the infringements of
* intellectual property rights of others.
*------------------------------------------------------------------------------
* File Name           : EEPROM-TEST.c
* Object              : main application written in C for AT91SAM7SE-EK
*
*                     : The Following code is for the following devices:
*                       - AT24C128/C256/C512/C1024
*
* Version | mm | dd | yy | author :
*  1.0      12   17   06    PFi   : Creation
*  1.1      14   08   07    PFi   : AT91F_AT24C_MemoryReset argument change.
*                                 : TWD & TWCK reversed.
*------------------------------------------------------------------------------
* 1)Description:
* ATMEL Two-wire Serial EEPROM AT24C test.
*
* This Demo code uses an ATMEL Two-wire Serial EEPROM connnected
* to the TWI port.
* Address used in this example with a AT24C1024 wit A1 = 1 (+VDD)
*
* ------| AT91 / IO  | AT24 / IO
* CLOCK | TWCK / PA3 | TWCK / 6
* DATA  | TWD / PA4  | TWD  / 5
*
*
* * WARNINGS ** :
* - No pull ups are mounted on the board on TWD and TWCK. 2.2K Pull up used
* - By default IAR compiler use the Tiny printf and scanf library format.
*   To be able to make print of float variables, you must set the library
*   format to large.
*
*  For further details check the following Application note
*  "AT91-AN01: Using the Two-wire interface (TWI) in
*  Master Mode on AT91SAM Microcontrollers."
*  www.atmel.com/dyn/products/app_notes.asp?family_id=605
*
* - By default IAR compiler use the Tiny printf and scanf library format.
*   To be able to make print of float variables, you must set the library
*   format to large.
*
* Remarks:
*
* This example runs in polling mode.and makes no use of IRQs. To have further
* details and implementation example about using IRQs, refer to any Interrupt
* project examples provided in Getting Started available on
* www.atmel.com/dyn/products/app_notes.asp?family_id=605*
******************************************************************************/
/* Include Standard LIB  files */
#include "atmel_at24c\at24c.h"
#include "common\init.h"
#include "common\lib_twi.h"

/* value of the PIT value to have 350ms @ 48 MHz
FFFFF -1 since real time out = PIV + 1 */
#define PIT_PIV_SECOND_VALUE       0xFFFFE

/* value of the PIT value to have 333ns @ 48 MHz
668 -1 since real time out = PIV + 1 */
#define PIT_PIV_MILI_SECOND_VALUE  0x667

/* value of the PIT value to have 333ns @ 48 MHz
3 -1 since real time out = PIV + 1 */
#define PIT_PIV_MICRO_SECOND_VALUE  0x2

#define TWI_BUS_CLOCK   400000 /* in Hz*/

volatile char message[96];      /* Buffer for sprintf */

/*-----------------------------------------------------------------------------
/ Wait function with the Periodic Interval Timer (PIT)
/ The wait time unit is the max PIT rollover value, i.e 350ms @ 48 MHz
/----------------------------------------------------------------------------*/
void AT91F_WaitSecond (unsigned int seconde)
{
   unsigned int PitStatus = 0;  /* Status register of the PIT */
   unsigned int PitLoop = 0;    /* Store the number of PIT Loop */

   AT91C_BASE_PITC->PITC_PIMR = AT91C_PITC_PITEN|PIT_PIV_SECOND_VALUE;

   for( PitLoop=0; PitLoop <(seconde*3);)    /* One PIT loop equals 333ms */
   {
    /* Wait for the PIT counter overflow occurs */
    while ((AT91C_BASE_PITC->PITC_PISR & AT91C_PITC_PITS)==0);
    /* Read the PIT Interval Value Reg to clear it for the next overflow */
    PitStatus = AT91C_BASE_PITC->PITC_PIVR ;
    /* dummy access to avoid IAR warning */
    PitStatus = PitStatus ;
    PitLoop++;
   }
}

/*-----------------------------------------------------------------------------
* Wait function with the Periodic Interval Timer (PIT)
* The wait time is from 1ms to 999ms.
-----------------------------------------------------------------------------*/
void AT91F_WaitMiliSecond (unsigned int MiliSeconds)
{
   unsigned int PitStatus = 0;     /* Status register of the PIT */
   unsigned int PitLoop = 0;    /* Store the number of PIT Loop */

   AT91C_BASE_PITC->PITC_PIMR = AT91C_PITC_PITEN|PIT_PIV_MILI_SECOND_VALUE;

   for( PitLoop=0; PitLoop <(MiliSeconds*3);)   /* One PIT loop equals 333ms */
   {
    /* Wait for the PIT counter overflow occurs */
    while ((AT91C_BASE_PITC->PITC_PISR & AT91C_PITC_PITS)==0);
    /* Read the PIT Interval Value Reg. to clear it for the next overflow */
    PitStatus = AT91C_BASE_PITC->PITC_PIVR ;
    /* dummy access to avoid IAR warning */
    PitStatus = PitStatus ;
    PitLoop++;
   }
}

/*-----------------------------------------------------------------------------
* Wait function with the Periodic Interval Timer (PIT)
* The wait time is from 1us to 999us.
-----------------------------------------------------------------------------*/
void AT91F_WaitMicroSecond (unsigned int MicroSecond)
{
   unsigned int PitStatus = 0;     /* Status register of the PIT */
   unsigned int PitLoop = 0;    /* Store the number of PIT Loop */

   AT91C_BASE_PITC->PITC_PIMR = AT91C_PITC_PITEN|PIT_PIV_MICRO_SECOND_VALUE;

   for( PitLoop=0; PitLoop < (MicroSecond*3);)   /* One PIT loop equals 333ns */
   {
    /* Wait for the PIT counter overflow occurs */
    while ((AT91C_BASE_PITC->PITC_PISR & AT91C_PITC_PITS)==0);
    /* Read the PIT Interval Value Reg. to clear it for the next overflow */
    PitStatus = AT91C_BASE_PITC->PITC_PIVR ;
    /* dummy access to avoid IAR warning */
    PitStatus = PitStatus ;
    PitLoop++;
   }
}

/*-----------------------------------------------------------------------------
* Function Name       : main
* Object              : Main function
* Input Parameters    : none
* Output Parameters   : TRUE
-----------------------------------------------------------------------------*/
int main( void )
{
    struct at24c ptAt24c1024 ;
    char TxBuffer[128] ;
    char RxBuffer[128] ;
    int BufferIndex;
    unsigned int PageCount;
    unsigned int error = 0 ;
    unsigned int i;

    /* Make a bus recovery sequence in case of the transmission has been
    broken due to a power loss or reset */
    AT91F_AT24C_MemoryReset (AT91C_BASE_PIOA,
                             AT91C_ID_PIOA,
                             AT91C_PA4_TWCK,
                             AT91C_PA3_TWD);

    /* TWI intialization @ TWI_BUS_CLOCK for all TWI devices
       connected on the bus */
    AT91F_TWI_Open(TWI_BUS_CLOCK);

    ptAt24c1024.PageSize = 256;
    ptAt24c1024.NumOfPage = 512;
    ptAt24c1024.EepromSize = 256*512;
    ptAt24c1024.SlaveAddress = AT24C_I2C_ADDRESS;
    ptAt24c1024.EepromName = AT24C1024;

    /* The following test writes the 512 pages. Data written within a page is
    the same for all bytes in that page.
    Page 0 filled with 0's, Page 1 filled with 1's etc...
    for (page 0 to page 255) Data = (0 to 255)
    for (page 256 to page 511) Data = (255 to 0) */

    /****************************** 1st part *******************************/
    /*       Before starting the test, fill the EEPROM with 0xFF           */
    /***********************************************************************/

    /* Initialize the Transmit buffer */
    for (BufferIndex=0; BufferIndex < sizeof(TxBuffer); BufferIndex++)
    {
      TxBuffer[BufferIndex] =0xFF ;
    }

    /* Clear the Receive buffer */
    for (BufferIndex=0; BufferIndex < sizeof(RxBuffer); BufferIndex++)
    {
      RxBuffer[BufferIndex] = 0 ;
    }

    /* Write the whole EEPROM with 0xFF */
    for (PageCount=0 ; PageCount < 512 ; PageCount++)
      AT91F_AT24C_WritePage (&ptAt24c1024, TxBuffer, sizeof TxBuffer,PageCount,0);

    /* Read the whole EEPROM to check 0xFF */
    for (PageCount=0 ; PageCount < 512 ; PageCount++)
    {
      AT91F_AT24C_ReadPage (&ptAt24c1024, RxBuffer, sizeof RxBuffer,PageCount,0);
      for (BufferIndex=0; BufferIndex < sizeof(TxBuffer); BufferIndex++)
      {
        if (TxBuffer[BufferIndex] != RxBuffer[BufferIndex])
          error++;
      }
    }

    /************************* 2nd part ************************************/
    /*                 Write all the pages with # values                   */
    /***********************************************************************/

    /* Initialize the Transmit buffer */
    for (PageCount=0 ; PageCount < 256 ; PageCount++)
    {
      /* for (page 0 to page 255) Data = (0 to 255) */
      for (BufferIndex=0; BufferIndex < sizeof(TxBuffer); BufferIndex++)
      {
        TxBuffer[BufferIndex] = PageCount ;     /* 0 to 255 */
      }

    AT91F_AT24C_WritePage (&ptAt24c1024, TxBuffer, sizeof TxBuffer,PageCount,0);
    }

    /*  for (page 256 to page 511) Data = (255 to 0) */
    for (PageCount=256, i=255 ; PageCount < 512 ; PageCount++, i--)
    {
      for (BufferIndex = (sizeof TxBuffer)-1 ; BufferIndex >= 0; BufferIndex--)
      {
        TxBuffer[BufferIndex] = i ;     /* 255 to 0 */
      }

    AT91F_AT24C_WritePage (&ptAt24c1024, TxBuffer, sizeof TxBuffer,PageCount,0);
    }

    /************************* 3rd part ************************************/
    /*                 Read all pages and compare                          */
    /***********************************************************************/

    /* for (page 0 to page 255) Data = (0 to 255) */
    for (PageCount=0 ; PageCount < 256 ; PageCount++)
    {
      for (BufferIndex=0; BufferIndex < sizeof(TxBuffer); BufferIndex++)
      {
        TxBuffer[BufferIndex] = PageCount ;
      }

     /* Read a complete page */
     AT91F_AT24C_ReadPage (&ptAt24c1024, RxBuffer, sizeof RxBuffer,PageCount,0);

     for (BufferIndex=0; BufferIndex < sizeof(TxBuffer); BufferIndex++)
     {
       if (TxBuffer[BufferIndex] !=  RxBuffer[BufferIndex])
         error++;
     }
    }

    /*  for (page 256 to page 511) Data = (255 to 0) */
    for (PageCount=256, i=255 ; PageCount < 512 ; PageCount++, i--)
    {
      for (BufferIndex = (sizeof TxBuffer)-1 ; BufferIndex >= 0; BufferIndex--)
      {
        TxBuffer[BufferIndex] = i ;     /* 255 to 0 */
      }

     /* Read a complete page */
     AT91F_AT24C_ReadPage (&ptAt24c1024, RxBuffer, sizeof RxBuffer,PageCount,0);

    for (BufferIndex = (sizeof TxBuffer)-1 ; BufferIndex >= 0; BufferIndex--)
     {
       if (TxBuffer[BufferIndex] != RxBuffer[BufferIndex])
         error++;
     }
    }

    /**********************************************************************/
    /* Test finished. Check the error variable. Should be 0 if no errors */
    /**********************************************************************/

    while (1);

}

