/*-----------------------------------------------------------------------------
*         ATMEL Microcontroller Software Support  -  ROUSSET  -
*------------------------------------------------------------------------------
* The software is delivered "AS IS" without warranty or condition of any
* kind, either express, implied or statutory. This includes without
* limitation any warranty or condition with respect to merchantability or
* fitness for any particular purpose, or against the infringements of
* intellectual property rights of others.
*------------------------------------------------------------------------------
* File Name           : GLCD-RTC-DS1337.c
* Object              : main application written in C.
*                     : Graphics LCD & Maxim/Dallas DS1337 RTC Demo Example
* Version | mm | dd | yy | author :
*  1.0      11   17   06    PFi   : Creation
*  1.1      12   27   06    PFi   : SQW output of the RTC connected to a PIO
*------------------------------------------------------------------------------
* 1)Description:
*
* This Demo uses the Dallas I2C RTC to have time and calendar connnected on
* the TWI port. The SWQ output of the RTC which generate a square wave signal
* of 1HZ, 4096HZ, 8192Hz or 32768HZ (default), is connected to a PIO line
* of the AT91 to get a IRQ to read the registers of the RTC.
*
* ------| AT91 / IO  | DS1337 / IO
* CLOCK | TWCK / PA3 | SCL / 6
* DATA  | TWD / PA4  | SDA / 5
* SQW   |    PA2     | SQW / 7
*
*
* * WARNINGS ** :
* - No pull ups are mounted on the board on TWD and TWCK. 2.2K Pull up used
* - By default IAR compiler use the Tiny printf and scanf library format.
*   To be able to make print of float variables, you must set the library
*   format to large.
*
* 3) Remarks:
*  For further details check the following Application note
*  "AT91-AN01: Using the Two-wire interface (TWI) in
*  Master Mode on AT91SAM Microcontrollers."
*  www.atmel.com/dyn/products/app_notes.asp?family_id=605
*
* - By default IAR compiler use the Tiny printf and scanf library format.
*   To be able to make print of float variables, you must set the library
*   format to large.
*
* This example runs in polling mode.and makes no use of IRQs. To have further
* details and implementation example about using IRQs, refer to any Interrupt
* project examples provided in Getting Started available on
* www.atmel.com/dyn/products/app_notes.asp?family_id=605
*
******************************************************************************/

/* Include Standard LIB  files */
#include "Board.h"
#include "common\lib_twi.h"
#include "time.h"
#include "rtc_ds1337\ds1337.h"
#include "glcd_batron\glcd_i2c_batron.h"
#include "math.h"

/* value of the PIT value to have 350ms @ 48 MHz */
#define PIT_PIV_SECOND_VALUE       0xFFFFF

/* value of the PIT value to have 333ns @ 48 MHz */
#define PIT_PIV_MILI_SECOND_VALUE  0x668

#define TWI_BUS_CLOCK   400000 /* in Hz*/

#define LOCAL_TIME_OFFSET   1   /* OffSet from GMT Time */

#define PIO_INTERRUPT_LEVEL 6

volatile int ReadRtc = 0 ;
volatile char message[96];      /* Buffer for sprintf */

/*-----------------------------------------------------------------------------
* Wait function with the Periodic Interval Timer (PIT)
* The wait time unit is the max PIT rollover value, i.e 350ms @ 48 MHz
*----------------------------------------------------------------------------*/
void AT91F_WaitSecond (unsigned int seconde)
{
   unsigned int PitStatus = 0;  /* Status register of the PIT */
   unsigned int PitLoop = 0;    /* Store the number of PIT Loop */

   AT91C_BASE_PITC->PITC_PIMR = AT91C_PITC_PITEN|PIT_PIV_SECOND_VALUE;

   for( PitLoop=0; PitLoop <(seconde*3);)    /* One PIT loop equals 333ms */
   {
    /* Wait for the PIT counter overflow occurs */
    while ((AT91C_BASE_PITC->PITC_PISR & AT91C_PITC_PITS)==0);
    /* Read the PIT Interval Value Reg to clear it for the next overflow */
    PitStatus = AT91C_BASE_PITC->PITC_PIVR ;
    /* dummy access to avoid IAR warning */
    PitStatus = PitStatus ;
    PitLoop++;
   }
}

/*-----------------------------------------------------------------------------
/ Wait function with the Periodic Interval Timer (PIT)
/ The wait time is from 1ms to 999ms.
/----------------------------------------------------------------------------*/
void AT91F_WaitMiliSecond (unsigned int MiliSeconds)
{
   unsigned int PitStatus = 0;     /* Status register of the PIT */
   unsigned int PitLoop = 0;    /* Store the number of PIT Loop */

   AT91C_BASE_PITC->PITC_PIMR = AT91C_PITC_PITEN|PIT_PIV_MILI_SECOND_VALUE;

   for( PitLoop=0; PitLoop <(MiliSeconds*3);)   /* One PIT loop equals 333ms */
   {
    /* Wait for the PIT counter overflow occurs */
    while ((AT91C_BASE_PITC->PITC_PISR & AT91C_PITC_PITS)==0);
    /* Read the PIT Interval Value Reg. to clear it for the next overflow */
    PitStatus = AT91C_BASE_PITC->PITC_PIVR ;
    /* dummy access to avoid IAR warning */
    PitStatus = PitStatus ;
    PitLoop++;
   }
}

/*-----------------------------------------------------------------------------
* Function Name       : void AT91F_Pio_C_Irq_Handler ( void )
* Object              : Irq C Handler
* Input Parameters    : none
* Output Parameters   : none
* Comments            :
-----------------------------------------------------------------------------*/
void AT91F_Pio_C_Irq_Handler ( void )
{
    int dummy, PinState;

    PinState = AT91F_PIO_GetInput(AT91C_BASE_PIOA) & AT91C_PIO_PA2 ;

    if (PinState)
      ReadRtc = 1;

    /* read the interrupt status reg of the PIO to acknoledge the IRQ */
    dummy =AT91C_BASE_PIOA->PIO_ISR;

    /* suppress the compilation warning */
    dummy =dummy;
}

/*-----------------------------------------------------------------------------
* Function Name       : main
* Object              : Main function
* Input Parameters    : none
* Output Parameters   : TRUE
-----------------------------------------------------------------------------*/
int main( void )
{
    time_t t = time(NULL);
    struct tm *SystemTime = localtime(&t);
    struct  DS1337 ptDS1337 ;

    /* Pointer to the AIC */
    AT91PS_AIC     pAic;

    /* Load System pAic Base address */
    pAic = AT91C_BASE_AIC;

    /* Enable User Reset and set its minimal assertion to 960 us */
    AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN |(0x4<<8) | (unsigned int)(0xA5<<24);

    /* First, enable the clock of the PIOA */
    AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_PIOA ) ;

    /* configure the PIO line corresponding to PA2
    to generate an IRQ on input change level */
    AT91F_PIO_CfgInput(AT91C_BASE_PIOA, SW4_MASK);

    /* configure the AIC for the PIO interrupt */
    AT91F_AIC_ConfigureIt (pAic,
                           AT91C_ID_PIOA,
                           PIO_INTERRUPT_LEVEL,
                           AT91C_AIC_SRCTYPE_HIGH_LEVEL,
                           AT91F_Pio_C_Irq_Handler);

    /* Enable the IRQ on that pin at  PIO level */
    AT91F_PIO_InterruptEnable(AT91C_BASE_PIOA,AT91C_PIO_PA2);

    /* set the interrupt by software */
    AT91F_AIC_EnableIt (pAic, AT91C_ID_PIOA);

    /*** TWI intialization @ TWI_BUS_CLOCK for all TWI devices connected
         on the bus ***/
    AT91F_TWI_Open(TWI_BUS_CLOCK);

    /* Set SQW output of the RTC to generate 1HZ for a time bease for
       register reading */
    AT91F_DS1337_SetControlRegister(DS1337_SQW_1HZ) ;

    /* Clear the LCD and put it in normal mode */
    AT91F_Glcd_ClearLcd();
    AT91F_Glcd_VideoMode(Normal);

    /*********************/
    /* Print Demo Header */
    /*********************/
    /* Print that string on the LCD. Max 19 characters per line/string. */
    AT91F_Glcd_PutString("*******************",LCD_PAGE_0,0);
    AT91F_Glcd_PutString("*     DS 1337     *",LCD_PAGE_1,0);
    AT91F_Glcd_PutString("* Real Time Clock *",LCD_PAGE_2,0);
    AT91F_Glcd_PutString("*      Demo       *",LCD_PAGE_3,0);
    AT91F_Glcd_PutString("*******************",LCD_PAGE_4,0);

    /* Wait 2 seconds between temperature sample */
      AT91F_WaitSecond (2);

    /* Clear the LCD and put it in normal mode */
    AT91F_Glcd_ClearLcd();
    AT91F_Glcd_VideoMode(Normal);

    /* Get Time and Calendar information from the system time at build time */
    ptDS1337.seconds = SystemTime->tm_sec;          /* seconds */
    ptDS1337.minutes = SystemTime->tm_min;          /* minnutes */

    /* force clock to 24 hour mode. Hours GMT Time. */
    ptDS1337.hours = ((SystemTime->tm_hour) & 0x3F)+LOCAL_TIME_OFFSET;
    ptDS1337.day = SystemTime->tm_wday;             /* day of the week */
    ptDS1337.date = SystemTime->tm_mday;            /* day of month */
    ptDS1337.month = (SystemTime->tm_mon);          /* montth */

    /* year + Century. Assume 20YY not 19YY, and ignore DS1337_BIT_CENTURY */
    ptDS1337.year = ((SystemTime->tm_year) -100);

    /* Set Time and Calendar into the RTC */
    AT91F_DS1337_SetDateTime(&ptDS1337);

    /* Clear the LCD and put it in normal mode */
    AT91F_Glcd_ClearLcd();
    AT91F_Glcd_VideoMode(Normal);

    AT91F_Glcd_PutString("*******************",LCD_PAGE_0,0);
    AT91F_Glcd_PutString("*******************",LCD_PAGE_4,0);

   while (1)
   {
     if (ReadRtc)
     {
     /* Get the Time and Date from the RTC */
      AT91F_DS1337_ReadDateTime(&ptDS1337);

      /* Print date on the LCD */
      AT91F_Glcd_ClearLcdPage(LCD_PAGE_1);
      sprintf((char*)message,"%s %d %s 20%02d", DayOfWeek[ptDS1337.day],
              ptDS1337.date,
              Month[ptDS1337.month],
              ptDS1337.year);

      AT91F_Glcd_PutString((char*)message,LCD_PAGE_1,0);

      /* Print time on the LCD. GMT and Local time */
      AT91F_Glcd_ClearLcdPage(LCD_PAGE_2);
      sprintf((char*)message,"GMT: %02d : %02d : %02d",
              (ptDS1337.hours)-LOCAL_TIME_OFFSET,
              ptDS1337.minutes, ptDS1337.seconds);

      AT91F_Glcd_PutString((char*)message,LCD_PAGE_2,0);

      AT91F_Glcd_ClearLcdPage(LCD_PAGE_3);
      sprintf((char*)message,"Local: %02d : %02d : %02d", ptDS1337.hours,
              ptDS1337.minutes,
              ptDS1337.seconds);

      AT91F_Glcd_PutString((char*)message,LCD_PAGE_3,0);
     }

     /* else do nothing */

   }
}

