/*-----------------------------------------------------------------------------
*         ATMEL Microcontroller Software Support  -  ROUSSET  -
*------------------------------------------------------------------------------
* The software is delivered "AS IS" without warranty or condition of any
* kind, either express, implied or statutory. This includes without
* limitation any warranty or condition with respect to merchantability or
* fitness for any particular purpose, or against the infringements of
* intellectual property rights of others.
*------------------------------------------------------------------------------
* File Name           : GLCD-TempSensor.c
* Object              : main application written in C.
*                     : Graphics LCD & LM75 Temperature Sensor Example
*
* Version | mm | dd | yy | author :
*  1.0      11   17   06    PFi   : Creation
*
*------------------------------------------------------------------------------
* 1)Description:
*
* This Demo uses the LM75 Temperature Sensor TWI Demo and the Graphic LCD
* module TWI Demo connnected on the TWI port.
*
* ------| AT91 / IO  | LM75 / IO | GLCD / IO
* CLOCK | TWCK / PA3 | SCL / 1   |  5
* DATA  | TWD / PA4  | SDA / 2   |  1
*
* * WARNINGS ** :
* No pull ups are mounted on the board on TWD and TWCK. 2.2K Pull up used
* By default IAR compiler use the Tiny printf and scanf library format.
* To be able to make print of float variables, you must set the library
* format to large.
*
* 3) Remarks:
*
*  For further details check the following Application note
*  "AT91-AN01: Using the Two-wire interface (TWI) in
*  Master Mode on AT91SAM Microcontrollers."
*  www.atmel.com/dyn/products/app_notes.asp?family_id=605
*
* - By default IAR compiler use the Tiny printf and scanf library format.
*   To be able to make print of float variables, you must set the library
*   format to large.
*
* This example runs in polling mode.and makes no use of IRQs. To have further
* details and implementation example about using IRQs, refer to any Interrupt
* project examples provided in Getting Started available on
* www.atmel.com/dyn/products/app_notes.asp?family_id=605
******************************************************************************/
// Include Standard LIB  files
#include "Board.h"
#include "common\lib_twi.h"
#include "lm75\lm75.h"
#include "glcd_batron\glcd_i2c_batron.h"
#include "math.h"

#define PIT_PIV_SECOND_VALUE       0xFFFFF  // value of the PIT value to have 350ms @ 48 MHz
#define PIT_PIV_MILI_SECOND_VALUE  0x668    // value of the PIT value to have 333ns @ 48 MHz

#define TWI_BUS_CLOCK   400000 /* in Hz*/

volatile char message[96];      // Buffer for AT91F_DBGU_PrintK function

/******************************************************************************
/ Wait function with the Periodic Interval Timer (PIT)
/ The wait time unit is the max PIT rollover value, i.e 350ms @ 48 MHz
******************************************************************************/
void AT91F_WaitSecond (unsigned int seconde)
{
   unsigned int PitStatus = 0;     // Status register of the PIT
   unsigned int PitLoop = 0;    // Store the number of PIT Loop

   AT91C_BASE_PITC->PITC_PIMR = AT91C_PITC_PITEN|PIT_PIV_SECOND_VALUE;

   for( PitLoop=0; PitLoop <(seconde*3);)        // One PIT loop equals 333ms
   {
    // Wait for the PIT counter overflow occurs
    while ((AT91C_BASE_PITC->PITC_PISR & AT91C_PITC_PITS)==0);
    // Read the PIT Interval Value Register to clear it for the next overflow
    PitStatus = AT91C_BASE_PITC->PITC_PIVR ;
    // dummy access to avoid IAR warning
    PitStatus = PitStatus ;
    PitLoop++;
   }
}

/******************************************************************************
/ Wait function with the Periodic Interval Timer (PIT)
/ The wait time is from 1ms to 999ms.
******************************************************************************/
void AT91F_WaitMiliSecond (unsigned int MiliSeconds)
{
   unsigned int PitStatus = 0;     // Status register of the PIT
   unsigned int PitLoop = 0;    // Store the number of PIT Loop

   AT91C_BASE_PITC->PITC_PIMR = AT91C_PITC_PITEN|PIT_PIV_MILI_SECOND_VALUE;

   for( PitLoop=0; PitLoop <(MiliSeconds*3);)        // One PIT loop equals 333ms
   {
    // Wait for the PIT counter overflow occurs
    while ((AT91C_BASE_PITC->PITC_PISR & AT91C_PITC_PITS)==0);
    // Read the PIT Interval Value Register to clear it for the next overflow
    PitStatus = AT91C_BASE_PITC->PITC_PIVR ;
    // dummy access to avoid IAR warning
    PitStatus = PitStatus ;
    PitLoop++;
   }
}


/******************************************************************************
/ Function Name       : main
/ Object              : Main function
/ Input Parameters    : none
/ Output Parameters   : None
******************************************************************************/
int main( void )
{
  float read;               /* Store the bytes received from the LM75 */
  float TempCelsius ;       /* Temperature in degree Celsius */
  float TempFahrenheit ;    /* Temperature in degree Fahrenhheit */

  /*** TWI intialization @ TWI_BUS_CLOCK for all TWI devices connected on the bus ***/
  AT91F_TWI_Open(TWI_BUS_CLOCK);

  AT91F_Glcd_ClearLcd();
  AT91F_Glcd_VideoMode(Normal);

  /**************************/
  /* Print LM75 Demo Header */
  /**************************/
  /* Print that string on the LCD. Max 19 characters per line/string */
  AT91F_Glcd_PutString("*******************",LCD_PAGE_0,0);
  AT91F_Glcd_PutString("*      LM 75      *",LCD_PAGE_1,0);
  AT91F_Glcd_PutString("*   Temperature   *",LCD_PAGE_2,0);
  AT91F_Glcd_PutString("*   Sensor Demo   *",LCD_PAGE_3,0);
  AT91F_Glcd_PutString("*******************",LCD_PAGE_4,0);

  /* Wait 3 seconds between temperature sample */
  AT91F_WaitSecond (3);

  /* Clear the LCD and put it in normal mode */
  AT91F_Glcd_ClearLcd();
  AT91F_Glcd_VideoMode(Normal);

  /***************************************************/
  /* Display LM75 registers contents and Temperature */
  /***************************************************/
  AT91F_LM75_SetConfigurationRegister(0x0);

  /* Read and Display in LCD Page 0 the LM75 Configuration Register */
  read = AT91F_LM75_ReadConfigurationRegister();
  sprintf((char*)message,"Conf Reg = 0x%X", (int)read);

  /* Print that string on the LCD. Max 19 characters per line/string. */
  AT91F_Glcd_PutString((char*)message,LCD_PAGE_0,0);

  /* Read and Display in LCD page 1 the LM75 Thyst register */
  read = AT91F_LM75_ReadThystRegister();
  sprintf((char*)message,"Thys Reg = %3.1fC", (float)read);

  /* Print that string on the LCD. Max 19 characters per line/string. */
  AT91F_Glcd_PutString((char*)message,LCD_PAGE_1,0);

  /* Read and Display in LCD page 2 the LM75 Tos register */
  read = AT91F_LM75_ReadTosRegister();
  sprintf((char*)message,"Tos Reg = %3.1fC", (float)read);
  AT91F_Glcd_PutString((char*)message,LCD_PAGE_2,0);
  AT91F_WaitSecond (2);

  /* Clear the LCD and put it in normal mode */
  AT91F_Glcd_ClearLcd();
  AT91F_Glcd_VideoMode(Normal);
  AT91F_Glcd_PutString("*******************",LCD_PAGE_0,0);
  AT91F_Glcd_PutString("*   Temperature   *",LCD_PAGE_1,0);
  AT91F_Glcd_PutString("*       is        *",LCD_PAGE_2,0);
  AT91F_Glcd_PutString("*                 *",LCD_PAGE_3,0);
  AT91F_Glcd_PutString("*******************",LCD_PAGE_4,0);

  /* Now read and display in LCD page 3 the Temp Register every 1 sec */
  while (1)
  {
    read = AT91F_LM75_ReadTosRegister();
    TempCelsius = AT91F_LM75_ReadTempRegister();

    /* To convert Celsius to Fahrenheit, multiply by 1.8 and add 32 degrees */
    TempFahrenheit = (TempCelsius * 1.8) + 32 ;
    sprintf((char*) message,"* %3.1fC - %3.1fF *",
            (float)TempCelsius,
            (float)TempFahrenheit);

    AT91F_Glcd_PutString((char*)message,LCD_PAGE_3,0);

    /* Wait 1 seconds between temperature sample */
    AT91F_WaitSecond (1);
  }

}
